#include "Weather.h"
#include "../SevenSegment.h"
#include "../Icons.h"
#include <Fonts/FreeSans9pt7b.h>
#include <Fonts/FreeSansBold9pt7b.h>
#include <Fonts/FreeSans12pt7b.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>

RTC_DATA_ATTR uint64_t WatchFacePages::Weather::m_lastSyncTime = 0;
RTC_DATA_ATTR int WatchFacePages::Weather::m_lastCalculatedDay = 0XFFFFFFFF;
RTC_DATA_ATTR float WatchFacePages::Weather::m_locationLat = 0.0f;
RTC_DATA_ATTR float WatchFacePages::Weather::m_locationLon = 0.0f;
RTC_DATA_ATTR char WatchFacePages::Weather::m_locationCity[128];

WatchFacePages::Weather::Weather(WatchyDisplay & display, WatchFeatures::WatchFeatures & features)
  : m_display(display), m_features(features)
{
}

void WatchFacePages::Weather::InitBoot()
{
}

void WatchFacePages::Weather::InitWake()
{
}

void WatchFacePages::Weather::DrawPage(bool partialRefresh)
{
  Serial.println("Weather:DrawPage");
  if (m_lastSyncTime == 0) {
    Resync();
  }

  m_display.setFullWindow();
  m_display.fillScreen(GxEPD_WHITE);
  m_display.setTextColor(GxEPD_BLACK);

  if (m_lastSyncTime == 0) {
    m_display.setFont(&FreeSans12pt7b);
    int16_t x, y;
    uint16_t w, h;
    m_display.getTextBounds("Have not synced", 0, 0, &x, &y, &w, &h);
    m_display.setCursor(DISPLAY_WIDTH / 2 - w / 2, 110);
    m_display.print("Have not synced");
    m_display.display(partialRefresh);
    return;
  }

  bool weatherOutdated = false;

  if (m_features.rtc.GetTimestamp() - m_lastSyncTime > 86400) {
    weatherOutdated = true;
  }

  m_display.setFont(&FreeSansBold9pt7b);
  int16_t x, y;
  uint16_t w, h;
  
  m_display.drawBitmap(5, 5, Icons::city, 29, 23, GxEPD_BLACK);

  m_display.setCursor(39, 21);
  m_display.print(m_locationCity);
  m_display.display(partialRefresh);
}

void WatchFacePages::Weather::Resync()
{
  uint64_t currentTime = m_features.rtc.GetTimestamp();

  if(m_lastSyncTime > 0 && currentTime - m_lastSyncTime < WEATHER_UPDATE_INTERVAL) {
    return;
  }

  if(!m_features.wifi.Connect()) {
    return;
  }

  HTTPClient client;
  client.setConnectTimeout(3000); // 3 second max timeout

  client.begin("http://ip-api.com/json/");
  int httpCode = client.GET();
  if (httpCode != 200) {
    m_features.wifi.Disconnect();
    return;
  }

  String payload = client.getString();
  client.end();

  DynamicJsonDocument json(1024);
  DeserializationError error = deserializeJson(json, payload);
  if (error) {
    m_features.wifi.Disconnect();
    return;
  }
  
  if (!json.containsKey("lat") || !json.containsKey("lon")) {
    m_features.wifi.Disconnect();
    return;
  }

  std::string location = json["city"].as<std::string>();
  location = location.substr(0, 127);
  memcpy(m_locationCity, location.c_str(), location.length());
  m_locationCity[location.length()] = '\0';
  m_locationLat = json["lat"];
  m_locationLon = json["lon"];
  m_features.wifi.Disconnect();

  m_lastSyncTime = currentTime;
}